
/******************************************************************************

Program to demonstrate the use servo motors with AVR Microcontrollers.

For More Details Visit: http://www.eXtremeElectronics.co.in

Copyright (c) 2008-2010
eXtreme Electronics, India

Servo Motor: Futaba sSERVO2_RIGHT_MOVE3
Servo Control PIN (white): To OC1A PIN
Crystal: 16MHz
LOW Fuse: 0xFF
HIGH Fuse: 0xC9

Compiler:avr-gcc toolchain
Project Manager/IDE: AVR Studio

                                     NOTICE
                           --------
NO PART OF THIS WORK CAN BE COPIED, DISTRIBUTED OR PUBLISHED WITHOUT A
WRITTEN PERMISSION FROM EXTREME ELECTRONICS INDIA. THE LIBRARY, NOR ANY PART
OF IT CAN BE USED IN COMMERCIAL APPLICATIONS. IT IS INTENDED TO BE USED FOR

HOBBY, LEARNING AND EDUCATIONAL PURPOSE ONLY. IF YOU WANT TO USE THEM IN 
COMMERCIAL APPLICATION PLEASE WRITE TO THE AUTHOR.


WRITTEN BY:
AVINASH GUPTA
me@avinashgupta.com

*******************************************************************************/
#include <avr/io.h>
#include <avr/interrupt.h>

#include <util/delay.h>

// LEDS that will light up when that servo is doing the beat
#define LED_PORT_SERVO1 PA0
#define LED_PORT_SERVO2 PA2

#define SERVO2_CONTROL_PORT PD4
#define SERVO1_CONTROL_PORT PD5

// DS04 servo. Using OCR1A
#define SERVO1_RIGHT_MOVE 268 //250 //255 //260 //250
#define SERVO1_LEFT_MOVE 450 //500 // 400 //550 //500
#define SERVO1_STOP 360 //375

// MG996r motor. Using OCR1B
#define SERVO2_RIGHT_MOVE 300
#define SERVO2_LEFT_MOVE 450

// Rotating mg996r motor 90 degrees left, 90 degrees right
#define SERVO2_FULL_RIGHT 170
#define SERVO2_FULL_LEFT 590

//#define SERVO1_MOVE_TIME 6
volatile int servo1_move_time = 6;
volatile int servo1_stay_time = 6;
volatile int total_move_time = 6;
//#define SERVO1_STAY_TIME 6
//#define TOTAL_MOVE_TIME 12

//

//Simple Wait Function
void Wait()
{
   uint8_t i;
   for(i=0;i<12;i++)
   {
      _delay_loop_2(0);
      _delay_loop_2(0);
      _delay_loop_2(0);
   }

}

void wait(int time)
{
   uint8_t i;
   for(i=0;i<time;i++)
   {
      _delay_loop_2(0);
      _delay_loop_2(0);
      _delay_loop_2(0);
   }

}

void calibrate_mg996r() {
   while (1) {
      OCR1B = SERVO2_FULL_RIGHT; // right
      Wait();
      OCR1B = SERVO2_FULL_LEFT; //left
      Wait();
      wait(100);
   }
}

void setup_ds04() {
   // stop ds04 for now
   OCR1A=SERVO1_STOP;
}

/* According to my dead batteries, interrupts and servos
 * may consume a lot of power. Protect your batteries.
 */
ISR(PCINT1_vect)
{
    if ((PINB & (1 << PB2)) == 0)
    {
      // button pressed
      calibrate_mg996r();
    }
}

void mg996r() {
   OCR1B=SERVO2_RIGHT_MOVE; //right mg996r
   Wait();

   OCR1B=SERVO2_LEFT_MOVE;  //left mg996r
   Wait();
}

void ds04() {
   OCR1A=SERVO1_RIGHT_MOVE; //full left
   wait(servo1_move_time);

   setup_ds04();
   wait(servo1_stay_time);

   OCR1A=SERVO1_LEFT_MOVE; //full right
   wait(servo1_move_time);
   setup_ds04();
   wait(servo1_stay_time);

   OCR1A=SERVO1_STOP; // stop
}

void simultaneously() {
   PORTA |= (1 << LED_PORT_SERVO2);
   OCR1B=SERVO2_RIGHT_MOVE; //right //mg996r
   PORTA |= (1 << LED_PORT_SERVO1);
   OCR1A=SERVO1_RIGHT_MOVE; //full left
   wait(servo1_move_time);
   setup_ds04();
   wait(total_move_time - servo1_move_time);
   //Wait();

   PORTA &= ~(1 << LED_PORT_SERVO2);
   OCR1B=SERVO2_LEFT_MOVE;  //full right // de fapt e left
   PORTA &= ~(1 << LED_PORT_SERVO1);
   OCR1A=SERVO1_LEFT_MOVE; //SERVO1_LEFT_MOVE;  //full right
   //Wait();
   wait(servo1_move_time);
   setup_ds04();
   wait(servo1_stay_time);
}

void simultaneously_loop() {
   while(1)
   {
      simultaneously();
   }
}

/*
 * If using in a loop, don't forget to setup ds04 before
 */
void one_after_the_other() {
   PORTA |= (1 << LED_PORT_SERVO2);
   mg996r();
   PORTA &= ~(1 << LED_PORT_SERVO2);

   PORTA |= (1 << LED_PORT_SERVO1);
   ds04();
   PORTA &= ~(1 << LED_PORT_SERVO1);
}

void one_after_the_other_loop() {
   while(1)
   {
      setup_ds04();
      one_after_the_other();
   }
}

/*
 * If using in a loop, don't forget to setup ds04 before
 */
void one_after_the_other_advanced() {
   // mg996r right
  PORTA |= (1 << LED_PORT_SERVO2);
   OCR1B=SERVO2_RIGHT_MOVE;
   // ds04 is waiting 6 seconds more now
   wait(6);
   // ds04 right
   PORTA |= (1 << LED_PORT_SERVO1);
   OCR1A=SERVO1_RIGHT_MOVE;
   wait(6);
   setup_ds04();
   
   //mg996r left
   PORTA &= ~(1 << LED_PORT_SERVO2);
   OCR1B=SERVO2_LEFT_MOVE;
   wait(6);
   //ds04 left
   PORTA &= ~(1 << LED_PORT_SERVO1);
   OCR1A=SERVO1_LEFT_MOVE;
   //mg996r is waiting 6 seconds more now
   wait(servo1_move_time);
   //wait(12);
   setup_ds04();
   wait(servo1_stay_time);

   //ds04 finished its step. stop moving while mg996r is doing right
   OCR1A=SERVO1_STOP;
}

void one_after_the_other_advanced_loop() {
   while(1)
   {
      setup_ds04();
      one_after_the_other_advanced();
   }
}

/*
 * If using in a loop, don't forget to setup ds04 before entering loop
 */
void one_after_the_other_advanced2() {
   // mg996r right. If in a loop, ds04 right and mg996r right simultaneously
  PORTA |= (1 << LED_PORT_SERVO2);
   OCR1B=SERVO2_RIGHT_MOVE;
   wait(12);

   // ds04 right
  PORTA |= (1 << LED_PORT_SERVO1);
   OCR1A=SERVO1_RIGHT_MOVE;
   //mg996r left
   PORTA &= ~(1 << LED_PORT_SERVO2);
   OCR1B=SERVO2_LEFT_MOVE;
   // mg996r moves up, ds04 moves down, simultaneously
   //wait(12);
   wait(servo1_move_time);
   setup_ds04();
   wait(total_move_time - servo1_move_time);
   
   //ds04 right. If in a loop, ds04 right and mg996r right simultaneously
   PORTA &= ~(1 << LED_PORT_SERVO1);
   OCR1A=SERVO1_LEFT_MOVE;
   wait(servo1_move_time);
   setup_ds04();
   wait(servo1_stay_time);
   //wait(12);
   setup_ds04();
}

void one_after_the_other_advanced2_loop() {
   while(1)
   {
      one_after_the_other_advanced2();
   }
}

void song1() {
   setup_ds04();
   mg996r();
   mg996r();
   ds04();
}

void song1_loop() {
   while(1)
   {
      song1();
   }
}

void buttons_one_after_the_other() {
   while(1) {
      if ((PINB & (1 << PB2)) == 0) {
         one_after_the_other();
      }
   }  
}

void buttons_simultaneously() {
   while(1) {
      if ((PIND & (1 << PD6)) == 0) {
         simultaneously();
      }
   }
}

void timer1_init() {
   //Configure TIMER1
   TCCR1A|=(1<<COM1A1)|(1<<COM1B1)|(1<<WGM11);        //NON Inverted PWM
   TCCR1B|=(1<<WGM13)|(1<<WGM12)|(1<<CS11)|(1<<CS10); //PRESCALER=64 MODE 14(FAST PWM)

   ICR1=4999;  //fPWM=50Hz (Period = 20ms Standard).
}

void IO_init() {
   /* LED-uri puse ca iesire */
    DDRA |= (1 << LED_PORT_SERVO1) | (1 << LED_PORT_SERVO2);

    /* LED-urile sunt stinse (punem pinii in starea LOW) */
    PORTA &= ~((1 << LED_PORT_SERVO1) | (1 << LED_PORT_SERVO2));
    //PORTD |= (1 << PD5) | (1 << PD7);

    /* butoanele sunt puse ca intrare, cu rezistenta de pull-up */
    /* PB2 and PD6 are pins for interrupts */
    DDRB &= ~(1 << PB2);
    DDRD &= ~(1 << PD6);
    PORTB |= (1 << PB2);
    PORTD |= (1 << PD6);


    // PWM Pins as Out
    // SERVO1 is controled by OCR1A
    // SERVO2 is controlled by OCR1B
    DDRD |= (1 << SERVO1_CONTROL_PORT) | (1 << SERVO2_CONTROL_PORT);

    /* intreruperile pentru butoane sunt activate */
/*    PCICR |= (1 << PCIE1) | (1 << PCIE3);
    PCMSK1 |= (1 << PCINT10);
    PCMSK3 |= (1 << PCINT30);*/

    // Activez PIN CHANGE INTERRUPT ENABLE pentru PCINT15:8 (portul B) si PCINT24:31 (portul D)
/*    PCICR |= (1 << PCIE1);
    // Activez intreruperea pe butoanele PB2 si PD6
    PCMSK1 |= (1 << PCINT10);*/
}

int main()
{
   // DON'T FORGET TO UNCOMMENT THIS IF USING INTERRUPTS FOR BUTTONS
   // sei();

   IO_init();
   timer1_init();

   //calibrate_mg996r();

   setup_ds04();
   // simultaneously_loop();
   one_after_the_other_loop();
   //one_after_the_other_advanced_loop();
   //one_after_the_other_advanced2_loop();
   //song1();

   return 0;
}

