#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>

/* trial and error limits for left and right :) */
#define LEFT_LIMIT 370
#define RIGHT_LIMIT 170

/* how much the servo moves at one step */
int step = 50;

/* duration of one period 50Mhz 20ms */
int max_iter = 4999;

/* number of interrupts */
int current_iter = 0;

/* servo initialize control */
int motor = LEFT_LIMIT;

/* OCR1A compare interruption */
ISR(TIMER1_COMPA_vect)
{
	/* increase iteration */
	current_iter += OCR1A;
	/* reset if period is reached */
	if(current_iter >= max_iter)
		current_iter = 0;
}

/* initializations */
void timer1_init()
{
	/* servo output pin */
	DDRB |= (1 << PB0);
	PORTB |= (1<<PB0);
	
	/* Setting the timer on CTC with top at OCRA, prescaler at 1 */
	TCCR1A |= (1<<COM1A1) | (1<<COM1B1)| (1<<WGM11) | (1<<WGM10);
	/* Prescaler 1 */
	TCCR1B |= (1<<WGM13) | (1<<WGM12)| (1<<CS10);
	
	/* How often the interrupt is triggered. The lower OCR1A, the more often */
	OCR1A = 1;
	
	/* set pins on HIGH */
	PORTA |= 0xFF;
	
	/* Enable OCR1A compare interruption */
	TIMSK1 = (1 << OCIE1A);
    
}

/* increase servo move value */
void right(volatile int *motor,int amount){
	if(*motor <= RIGHT_LIMIT)
		*motor += step;
}

/* decrease servo move value */
void left(volatile int *motor,int amount)
{
	if(*motor >= LEFT_LIMIT)
		*motor -= step;
}

int main(void)
{
	/* enable global interrupts */
    	sei();
	
	/* init timer1 */
	timer1_init();
	
	/* debug */
	int dir = 1;
	int btn_pressed = 0;

	while(1){
		btn_pressed = 0;

		PORTA &= ~(1<<PA5);
		PORTA |= (1<<PA5);

		/* if button has been pressed, adjust servo angle */
		if((PINA & (1<<PA5)) == 0)
		{
			right(&motor,step);
			btn_pressed = 1;
		} else if ((PINA & (1<<PA6)) == 0)
		{
			left(&motor,step);
			btn_pressed = 1;			
		}
	}
    return 0;
}

